@media print {
    @page {
        /* 15cm ereader/kindle */
        padding: 0;
        margin: 5mm 5mm;
        size: 90mm 120mm;
    }
    body {
        /* ereader/kindle */
        font-size: 8pt;
        line-height: 1.5em;
        letter-spacing: 0;
    }
    img {
        max-width: calc(90mm - 5mm);
        max-height: calc(120mm - 5mm);
    }

    div.codehilite pre {
        font-size: 6.7pt;
        letter-spacing: 0em;
        margin-left: 0em;
    }
    blockquote p, .admonition p, blockquote li, .admonition li {
        line-height: 1.4em
    }

    @page :first {
        @top-left     {content: "."; color: #FEFEFE; font-size: 4pt;margin-bottom: -3em;}
        @top-right    {content: "."; color: #FEFEFE; font-size: 4pt;margin-bottom: -3em;}
    }
    @page {
        /* Render faint text in corners to deal with automatic page cropping. */
        @top-left     {content: "."; color: #FEFEFE; font-size: 4pt;margin-bottom: -3em;}
        @top-right    {content: "."; color: #FEFEFE; font-size: 4pt;margin-bottom: -3em;}
        @bottom-left  {content: "`"; color: #FEFEFE; font-size: 4pt;margin-top: -5em;}
        @bottom-right {content: "´"; color: #FEFEFE; font-size: 4pt;margin-top: -5em;}
    }
}
