@media print {
    @page {
        /* 1of2col-A4 - for 2 column paper layout */
        padding: 0;
        size: 105mm 297mm;
    }

    img {
        max-width: calc(105mm - 16mm);
        max-height: calc(297mm - 32mm);
    }
}