@media print {
    @page {
        /* 1of2col-Letter - for 2 column paper layout */
        padding: 0;
        size: 4.25in 11in;
    }

    img {
        max-width: calc(4.25in - 16mm);
        max-height: calc(11in - 32mm);
    }
}