code {
    font-family: "lp-iosevka", "Iosevka Term SS05", monospace;
}

p > code {background-color: #CCC4;}

pre {
    font-family: "lp-iosevka", "Iosevka Term SS05", monospace;
    line-height: 1.2em;
}

@media print {
    pre {
        line-height: 1.2em;
    }
}

div.codehilite pre {
    display: inline-block;
    border: 1px solid #888;
    border-width: 0 1px 0 0;
    margin: 0 auto;
}

span.lineno {
    color: #666;
    display: inline-block;
    width: 1.3em;
    padding-right: 0.3em;
    margin-right: 0.5em;
    border-right: 1px solid #888;
    text-align: right;
    -webkit-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

.codehilite .l {  color: #060; }  /* Literal */
.codehilite .c { font-style: italic; } /* Comment */
.codehilite .err { color: #F00 } /* Error */
.codehilite .k {  color: #222; } /* Keyword */
.codehilite .ch { color: #444; } /* Comment.Hashbang */
.codehilite .cm { color: #444; } /* Comment.Multiline */
.codehilite .cpf { color: #444; } /* Comment.PreprocFile */
.codehilite .c1 { color: #444; } /* Comment.Single */
.codehilite .cs { color: #444; } /* Comment.Special */
.codehilite .ge { font-style: italic; } /* Generic.Emph */
.codehilite .gh {  color: #000; } /* Generic.Heading */
.codehilite .gp {  color: #000; } /* Generic.Prompt */
.codehilite .gs {  color: #000; } /* Generic.Strong */
.codehilite .gu {  color: #000; } /* Generic.Subheading */
.codehilite .kc {  color: #000; } /* Keyword.Constant */
.codehilite .kd {  color: #000; } /* Keyword.Declaration */
.codehilite .kn { color: #000; } /* Keyword.Namespace */
.codehilite .kr {  color: #000; } /* Keyword.Reserved */
.codehilite .s { } /* Literal.String */
.codehilite .mi { color: #060; } /* */
.codehilite .nc {  color: #000; } /* Name.Class */
.codehilite .ni {  color: #222;  } /* Name.Entity */
.codehilite .ne {  color: #222;  } /* Name.Exception */
.codehilite .nn {  color: #000; } /* Name.Namespace */
.codehilite .nt {  color: #222;  } /* Name.Tag */
.codehilite .o {  color: #800; } /* Operator */
.codehilite .ow {  color: #800; } /* Operator.Word */
.codehilite .sa { color: #006; } /* Literal.String.Affix */
.codehilite .sb { color: #006; } /* Literal.String.Backtick */
.codehilite .sc { color: #006; } /* Literal.String.Char */
.codehilite .dl { color: #006; } /* Literal.String.Delimiter */
.codehilite .sd { color: #006; font-style: italic; } /* Literal.String.Doc */
.codehilite .s1 { color: #006; } /* Literal.String.Single */
.codehilite .s2 { color: #006; } /* Literal.String.Double */
.codehilite .se { color: #006; } /* Literal.String.Escape */
.codehilite .sh { color: #006; } /* Literal.String.Heredoc */
.codehilite .si { color: #006; } /* Literal.String.Interpol */
.codehilite .sx { color: #006; } /* Literal.String.Other */
.codehilite .sr { color: #006; } /* Literal.String.Regex */
.codehilite .ss { color: #006; } /* Literal.String.Symbol */
.codehilite .bp { color: #060; } /* Literal.String.Symbol */



@media screen {

.dark span.lineno {color: #AAA;}

.dark .codehilite .l {  color: #FFF; }  /* Literal */
.dark .codehilite .c { font-style: italic; } /* Comment */
.dark .codehilite .err { color: #F00 } /* Error */
.dark .codehilite .k {  color: #DDD; } /* Keyword */
.dark .codehilite .ch { color: #AAA; } /* Comment.Hashbang */
.dark .codehilite .cm { color: #AAA; } /* Comment.Multiline */
.dark .codehilite .cpf { color: #AAA; } /* Comment.PreprocFile */
.dark .codehilite .c1 { color: #AAA; } /* Comment.Single */
.dark .codehilite .cs { color: #AAA; } /* Comment.Special */
.dark .codehilite .ge { font-style: italic; } /* Generic.Emph */
.dark .codehilite .gh {  color: #FFF; } /* Generic.Heading */
.dark .codehilite .gp {  color: #FFF; } /* Generic.Prompt */
.dark .codehilite .gs {  color: #FFF; } /* Generic.Strong */
.dark .codehilite .gu {  color: #FFF; } /* Generic.Subheading */
.dark .codehilite .kc {  color: #FFF; } /* Keyword.Constant */
.dark .codehilite .kd {  color: #FFF; } /* Keyword.Declaration */
.dark .codehilite .kn { color: #FFF; } /* Keyword.Namespace */
.dark .codehilite .kr {  color: #FFF; } /* Keyword.Reserved */
.dark .codehilite .s { } /* Literal.String */
.dark .codehilite .mi { color: #FFF; } /* */
.dark .codehilite .nc {  color: #FFF; } /* Name.Class */
.dark .codehilite .ni {  color: #DDD;  } /* Name.Entity */
.dark .codehilite .ne {  color: #DDD;  } /* Name.Exception */
.dark .codehilite .nn {  color: #FFF; } /* Name.Namespace */
.dark .codehilite .nt {  color: #DDD;  } /* Name.Tag */
.dark .codehilite .o {  color: #F99; } /* Operator */
.dark .codehilite .ow {  color: #F99; } /* Operator.Word */
.dark .codehilite .sa { color: #BBF; } /* Literal.String.Affix */
.dark .codehilite .sb { color: #BBF; } /* Literal.String.Backtick */
.dark .codehilite .sc { color: #BBF; } /* Literal.String.Char */
.dark .codehilite .dl { color: #BBF; } /* Literal.String.Delimiter */
.dark .codehilite .sd { color: #BBF; font-style: italic; } /* Literal.String.Doc */
.dark .codehilite .s1 { color: #BBF; } /* Literal.String.Single */
.dark .codehilite .s2 { color: #BBF; } /* Literal.String.Double */
.dark .codehilite .se { color: #BBF; } /* Literal.String.Escape */
.dark .codehilite .sh { color: #BBF; } /* Literal.String.Heredoc */
.dark .codehilite .si { color: #BBF; } /* Literal.String.Interpol */
.dark .codehilite .sx { color: #BBF; } /* Literal.String.Other */
.dark .codehilite .sr { color: #BBF; } /* Literal.String.Regex */
.dark .codehilite .ss { color: #BBF; } /* Literal.String.Symbol */
.dark .codehilite .bp { color: #FFF; } /* Literal.String.Symbol */

}


@media print {

/* TODO: Maybe not disable colouring totally for print? */
.dark .codehilite .err { color: #400 } /* Error */

.codehilite .nn { color: #000; } /* Name.Namespace */
.codehilite .o  { color: #000; } /* Operator */
.codehilite .ow { color: #000; } /* Operator.Word */
.codehilite .sa { color: #000; } /* Literal.String.Affix */
.codehilite .sb { color: #000; } /* Literal.String.Backtick */
.codehilite .sc { color: #000; } /* Literal.String.Char */
.codehilite .dl { color: #000; } /* Literal.String.Delimiter */
.codehilite .sd { color: #000; font-style: italic; } /* Literal.String.Doc */
.codehilite .s1 { color: #000; } /* Literal.String.Single */
.codehilite .s2 { color: #000; } /* Literal.String.Double */
.codehilite .se { color: #000; } /* Literal.String.Escape */
.codehilite .sh { color: #000; } /* Literal.String.Heredoc */
.codehilite .si { color: #000; } /* Literal.String.Interpol */
.codehilite .sx { color: #000; } /* Literal.String.Other */
.codehilite .sr { color: #000; } /* Literal.String.Regex */
.codehilite .ss { color: #000; } /* Literal.String.Symbol */

.codehilite .l  { color: #000; }  /* Literal */
.codehilite .mi { color: #000; } /* */
.codehilite .bp { color: #000; } /* Literal.String.Symbol */

.codehilite .ni { color: #000;  } /* Name.Entity */
.codehilite .ne { color: #000;  } /* Name.Exception */
.codehilite .nt { color: #000;  } /* Name.Tag */

}

