
/* enriqueta-regular - latin-ext_latin */
@font-face {
  font-family: 'lp-enriqueta';
  font-style: normal;
  font-weight: 400;
  src: local(''),
       url('/static/enriqueta-v10-latin-ext_latin-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-regular.woff') format('woff'), /* Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-regular.ttf') format('truetype'); /* Legacy iOS */
}
/* enriqueta-600 - latin-ext_latin */
@font-face {
  font-family: 'lp-enriqueta';
  font-style: normal;
  font-weight: 600;
  src: local(''),
       url('/static/enriqueta-v10-latin-ext_latin-600.woff2') format('woff2'), /* Super Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-600.woff') format('woff'), /* Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-600.ttf') format('truetype'); /* Legacy iOS */
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 400;
  font-style: normal;
  font-display: swap;
  src: url('/static/iosevka-term-ss05-regular.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-regular.woff') format('woff'),
       url('/static/iosevka-term-ss05-regular.ttf') format('truetype');
}


/* bitter-regular - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: normal;
  font-display: block;
  font-weight: 400;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-regular.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-regular.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-regular.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
/* bitter-italic - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: italic;
  font-display: block;
  font-weight: 400;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-italic.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-italic.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-italic.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
/* bitter-600 - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: bold;
  font-display: block;
  font-weight: 600;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-600.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-600.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-600.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
