/* Via https://google-webfonts-helper.herokuapp.com/
 *
 * Note that the lp- prefix is so that we give preference to the
 * font files we serve, rather than to fonts installed on the system
 * that might have the same name.
 */


/* gelasio-regular - latin-ext_latin */
@font-face {
  font-family: 'lp-gelasio';
  font-style: normal;
  font-weight: 400;
  src: local('Gelasio Regular'), local('Gelasio-Regular'),
       url('/static/gelasio-v1-latin-ext_latin-regular.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
       url('/static/gelasio-v1-latin-ext_latin-regular.woff') format('woff'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}

/* gelasio-italic - latin-ext_latin */
@font-face {
  font-family: 'lp-gelasio';
  font-style: italic;
  font-weight: 400;
  src: local('Gelasio Italic'), local('Gelasio-Italic'),
       url('/static/gelasio-v1-latin-ext_latin-italic.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
       url('/static/gelasio-v1-latin-ext_latin-italic.woff') format('woff'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}

/* gelasio-700 - latin-ext_latin */
@font-face {
  font-family: 'lp-gelasio';
  font-style: normal;
  font-weight: 700;
  src: local('Gelasio Bold'), local('Gelasio-Bold'),
       url('/static/gelasio-v1-latin-ext_latin-700.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
       url('/static/gelasio-v1-latin-ext_latin-700.woff') format('woff'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}

/* gelasio-700italic - latin-ext_latin */
@font-face {
  font-family: 'lp-gelasio';
  font-style: italic;
  font-weight: 700;
  src: local('Gelasio Bold Italic'), local('Gelasio-BoldItalic'),
       url('/static/gelasio-v1-latin-ext_latin-700italic.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
       url('/static/gelasio-v1-latin-ext_latin-700italic.woff') format('woff'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}

/* enriqueta-regular - latin-ext_latin */
@font-face {
  font-family: 'lp-enriqueta';
  font-style: normal;
  font-weight: 400;
  src: local(''),
       url('/static/enriqueta-v10-latin-ext_latin-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-regular.woff') format('woff'), /* Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-regular.ttf') format('truetype'); /* Legacy iOS */
}
/* enriqueta-600 - latin-ext_latin */
@font-face {
  font-family: 'lp-enriqueta';
  font-style: normal;
  font-weight: 600;
  src: local(''),
       url('/static/enriqueta-v10-latin-ext_latin-600.woff2') format('woff2'), /* Super Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-600.woff') format('woff'), /* Modern Browsers */
       url('/static/enriqueta-v10-latin-ext_latin-600.ttf') format('truetype'); /* Legacy iOS */
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 400;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-regular.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-regular.woff') format('woff'),
       url('/static/iosevka-term-ss05-regular.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 400;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-oblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-oblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-oblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 400;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-italic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-italic.woff') format('woff'),
       url('/static/iosevka-term-ss05-italic.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 500;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-medium.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-medium.woff') format('woff'),
       url('/static/iosevka-term-ss05-medium.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 500;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-mediumoblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-mediumoblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-mediumoblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 500;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-mediumitalic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-mediumitalic.woff') format('woff'),
       url('/static/iosevka-term-ss05-mediumitalic.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 600;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-semibold.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-semibold.woff') format('woff'),
       url('/static/iosevka-term-ss05-semibold.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 600;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-semiboldoblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-semiboldoblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-semiboldoblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 600;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-semibolditalic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-semibolditalic.woff') format('woff'),
       url('/static/iosevka-term-ss05-semibolditalic.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 700;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-bold.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-bold.woff') format('woff'),
       url('/static/iosevka-term-ss05-bold.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 700;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-boldoblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-boldoblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-boldoblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 700;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-bolditalic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-bolditalic.woff') format('woff'),
       url('/static/iosevka-term-ss05-bolditalic.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 800;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-extrabold.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-extrabold.woff') format('woff'),
       url('/static/iosevka-term-ss05-extrabold.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 800;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-extraboldoblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-extraboldoblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-extraboldoblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 800;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-extrabolditalic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-extrabolditalic.woff') format('woff'),
       url('/static/iosevka-term-ss05-extrabolditalic.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 900;
  font-style: normal;
  font-display: block;
  src: url('/static/iosevka-term-ss05-heavy.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-heavy.woff') format('woff'),
       url('/static/iosevka-term-ss05-heavy.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 900;
  font-style: oblique;
  font-display: block;
  src: url('/static/iosevka-term-ss05-heavyoblique.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-heavyoblique.woff') format('woff'),
       url('/static/iosevka-term-ss05-heavyoblique.ttf') format('truetype');
}

@font-face {
  font-family: 'lp-iosevka';
  font-weight: 900;
  font-style: italic;
  font-display: block;
  src: url('/static/iosevka-term-ss05-heavyitalic.woff2') format('woff2'),
       url('/static/iosevka-term-ss05-heavyitalic.woff') format('woff'),
       url('/static/iosevka-term-ss05-heavyitalic.ttf') format('truetype');
}


/* NOTE (mb): The only thing I don't like about Bitter, is that it's too wide,
 *    otherwise i'd prefer it over Gelasio, Georgia for body text.
 */

/* bitter-300 - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: normal;
  font-display: block;
  font-weight: 300;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-300.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-300.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-300.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}

/* bitter-regular - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: normal;
  font-display: block;
  font-weight: 400;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-regular.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-regular.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-regular.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
/* bitter-italic - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: italic;
  font-display: block;
  font-weight: 400;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-italic.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-italic.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-italic.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
/* bitter-600 - latin-ext_latin */
@font-face {
  font-family: 'lp-bitter';
  font-style: bold;
  font-display: block;
  font-weight: 600;
  src: local(''),
      url('/static/bitter-v17-latin-ext_latin-600.woff2') format('woff2'), /* Chrome 26+, Opera 23+, Firefox 39+ */
      url('/static/bitter-v17-latin-ext_latin-600.woff') format('woff'),
      url('/static/bitter-v17-latin-ext_latin-600.ttf') format('truetype'); /* Chrome 6+, Firefox 3.6+, IE 9+, Safari 5.1+ */
}
