#!/usr/bin/env python

# This file is part of the litprog project
# https://github.com/litprog/litprog
#
# Copyright (c) 2018-2021 Manuel Barkhau (mbarkhau@gmail.com) - MIT License
# SPDX-License-Identifier: MIT

import litprog.cli

if __name__ == '__main__':
    litprog.cli.cli()
